unit TestDM;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  TabdPalmConduit, TabdPalmFields;

{
  This demo assumes that there is a conduit entry created with :
      Type       = Application
      Conduit    = DelphiConduit.dll
      Creator Id = memo  (you should use the creator id of an application on
                          your Palm that is not already used by another conduit)

  I use the CondCfg.exe utility that comes with the CDK to create the new entry.

  Got to Project\Options... and then the Directories/Conditional tab and
  check that the Output Directory is the Palm directory (c:\Program Files\Palm
  by default).
}


type
  TDataModule1 = class(TDataModule)
    PalmDB: TtabdConduitDb;
    tabdConduit1: TtabdConduit;
    procedure tabdConduit1OpenConduit(var Error: Integer);
    procedure PalmDBTestFieldExistsForRead(Sender: TtabdConduitDb;
      Field: TtabdPalmField; ArrayIndex: Integer;
      var FieldExists: Boolean);
    procedure PalmDBTestFieldExistsForWrite(Sender: TtabdConduitDb;
      Field: TtabdPalmField; ArrayIndex: Integer;
      var FieldExists: Boolean);
  private
  public
  end;


var
  DataModule1: TDataModule1;

implementation

{$R *.DFM}

uses
  Math;

function MidStr(const Str : string;const Start,Size : Word) : string;
  var
    RealStart : Integer;
begin
  RealStart := Start;

  if RealStart < 1 then begin
    RealStart := 1;
  end;

  Result := Copy(Str,RealStart,Size);
end;


procedure TDataModule1.tabdConduit1OpenConduit(var Error: Integer);
  var
    Buf    : string;
    Count  : Integer;

  function GetPhoneMeaning(PhoneField : Integer) : string;
    const
      PhoneNames : array[0..7] of string = ('Work','Home','Fax','Other','EMail',
                                            'Main','Pager','Mobile');

    var
      TempLong : Longint;
  begin
    TempLong := PalmDb.FieldByName('Options').Value;
    Result   := PhoneNames[StrToInt(MidStr(Format('%8x',[TempLong]),9 - PhoneField,1))];
  end;

begin
  Count := 0;

  try
    tabdConduit1.AddLogEntry('In OpenConduit');
    PalmDb.Open;

    // Read in all the address book entries
    while not PalmDb.Eof do begin
      Buf := VarToStr(PalmDb.FieldByName('FirstName').Value) + #13#10 +
             VarToStr(PalmDb.FieldByName('Name').Value) + #13#10 +
             VarToStr(PalmDb.FieldByName('Title').Value) + #13#10 +
             VarToStr(PalmDb.FieldByName('Company').Value) + #13#10 +
             GetPhoneMeaning(1) + ' : ' + VarToStr(PalmDb.FieldByName('Phone1').Value) + #13#10 +
             GetPhoneMeaning(2) + ' : ' + VarToStr(PalmDb.FieldByName('Phone2').Value) + #13#10 +
             GetPhoneMeaning(3) + ' : ' + VarToStr(PalmDb.FieldByName('Phone3').Value) + #13#10 +
             GetPhoneMeaning(4) + ' : ' + VarToStr(PalmDb.FieldByName('Phone4').Value) + #13#10 +
             GetPhoneMeaning(5) + ' : ' + VarToStr(PalmDb.FieldByName('Phone5').Value);

      Inc(Count);

      // Display the first 10 entries for debugging purposes
      if Count < 10 then begin
        ShowMessage(Buf);
      end;

      PalmDb.Next;
      tabdConduit1.YieldCycles(10);
    end;

  finally
    PalmDb.Close;
  end;
end;


procedure TDataModule1.PalmDBTestFieldExistsForRead(
  Sender: TtabdConduitDb; Field: TtabdPalmField; ArrayIndex: Integer;
  var FieldExists: Boolean);
  var
    PowerOfTwo : Longint;
begin
  if Field.Index <= 2 then begin
    FieldExists := True;
    Exit;
  end;

  PowerOfTwo  := Trunc(IntPower(2,Field.Index - 3));
  FieldExists := Sender.FieldByName('Flags').Value and PowerOfTwo;
end;


procedure TDataModule1.PalmDBTestFieldExistsForWrite(
  Sender: TtabdConduitDb; Field: TtabdPalmField; ArrayIndex: Integer;
  var FieldExists: Boolean);
  var
    PowerOfTwo : Longint;
begin
  if Field.Index <= 2 then begin
    FieldExists := True;
    Exit;
  end;

  FieldExists := not VarIsEmpty(Field.Value);

  if FieldExists then begin
    PowerOfTwo := Trunc(IntPower(2,Field.Index - 3));
    Sender.FieldByName('Flags').AsInteger := Sender.FieldByName('Flags').AsInteger or PowerOfTwo;
  end;
end;


end.

